#!/bin/bash
# Script bash to copy the needed compiler library for the Ac6 Rootfs Project
# Do executed on the project root directory
#
# REQUIRED VARIABLES :
# compiler_prefix : the GCC-based compiler prefix
#   
# Do not modify !
#
# Note that, thanks to shared library handling and auto_depends , this step 
# should no more be needed; so there should be no library copy displayed.
#


here=$(pwd)
basedir=$(dirname "$0")
rootfsDir=$here/output/rootfs
sysrootDir=$SYSROOT_LOC

bash="/bin/bash"
sed="/bin/sed"
grep="/bin/grep"
readlink="/bin/readlink"
crossldd="$basedir/cross-ldd"
 
prefix=${arch_compiler_prefix:-arm-linux-gnueabi}
readelf="/usr/bin/readelf"

declare -a sourceDirs=("bin" "sbin" "usr/bin" "usr/sbin" "lib" "usr/lib")
declare -a listElf

mkdir -p ${rootfsDir}/lib

# Check if the readelf or cross-readelf is existing
crossprefix="$( basename "${prefix}" )"

gcc="${prefix}-gcc"
if [ ! -x "$( which "${gcc}" )" ]; then
        gcc="${crossprefix}-gcc"
fi
if [ ! -x "$( which "${gcc}" )" ]; then
        echo "Can't execute '${prefix}-gcc' nor '${gcc}'"
        exit 1
fi

readelf="${prefix}-readelf"
if [ ! -x "$( which "${readelf}" )" ]; then
        readelf="${crossprefix}-readelf"
fi
if [ ! -x "$( which "${readelf}" )" ]; then
        readelf="readelf"
fi
if [ ! -x "$( which "${readelf}" )" ]; then
        echo "Can't execute '${prefix}-readelf' nor '${crossprefix}-readelf or readelf'"
        exit 1
fi

# Check if the file in argument is an ELF file then add it in a list
add_elf() {
	local elfFile=$1
	local type=$( ${readelf} -h ${elfFile} 2> /dev/null | ${grep} Class | ${sed} -r -e 's/^.*Class.*:[[:space:]]+(.*)$/\1/;' )
	if [[ $type == *"ELF"* ]]; then
		listElf=(${listElf[@]} ${elfFile})
	fi
}

# Get the list of ELF files from the input directory 
get_elf_files() {	
	local dir=$1	
	# If the source directory exists		
	if [ -d $dir ]; then
		for file in $( find $dir -type f )
		do			
			add_elf $file
		done
	fi
}

# Copy the lib C if does not exist
copy_libc() {
	local dest=$1
	local source=$2
	local linkSource="$(${readlink} -f ${source})"

	if [ ! -s "${dest}" ]; then
		if [ -L "${source}" ]; then
			libName=$(basename $linkSource)
			destLib="$(dirname $dest)/$libName"
			echo "cp -a ${linkSource} ${destLib}"
			echo "ln -nfs ${libName} ${dest}"
			cp -a ${linkSource} ${destLib}
			ln -nfs ${libName} ${dest}
		else
			echo "cp -a ${source} ${dest}"
			cp -a ${source} ${dest}
		fi	
	fi
}

# Copy the required C libraries for the file in argument
copy_libsc() {
	local elfFile=$1
	${bash} ${crossldd} -m --prefix=$prefix --root=$rootfsDir $elfFile | \
	while read line
	do
		# Skip the virtual syscall page
		if echo "${line}" | grep -q "linux-vdso"; then continue; fi 
		if [ ! -z "${line}" ]; then
			source=$(echo $line | awk -F' <= ' '{ print $1 }')	
			dest=$(echo $line | awk -F' <= ' '{ print $2 }')
			copy_libc $source $dest
		fi
	done
}

# If there is a ld.so.conf file and/or ld.so.conf.d directory read them
if [ -s $rootfsDir/etc/ld.so.conf ]; then
	grep -v '^include ' $rootfsDir/etc/ld.so.conf > /tmp/libdirs.$$
	for file_spec in $(sed -n -e 's/^include  *//p'); do
		for file in $(echo $file_spec); do
			if [ -s $file ]; then
				cat $file >> /tmp/libdirs.$$
			fi
		done
	done
	while read dir; do
		sourceDir[]="$dir"
	done < /tmp/libdirs.$$
fi

# If the cross-ldd script exists
if [ -f ${crossldd} ]; then
	
 	# Search every ELF files from source directories
	for sourceDir in ${sourceDirs[*]}
	do
		case $sourceDir in
			/*)	dir=${rootfsDir}$sourceDir	;;
			*)	dir=${rootfsDir}/$sourceDir	;;
		esac
		get_elf_files $dir
	done

	for elfFile in ${listElf[*]}
	do	
		copy_libsc $elfFile
	done
fi


