#!/bin/bash
# Script to run create image step
# Under fakeroot
# minimal-rootfs/internal
# step: following task: BUILD
# step: Previous tasks: COPY LIBRARIES

here=$(pwd)
basedir=$(dirname "$0")
outputDir=$here/output
rootfsDir=$outputDir/rootfs
imagesDir=$outputDir/images
tmpDir=/tmp/create_image.$$
mkdir -p $imagesDir $tmpDir
trap "rm -rf $tmpDir" EXIT

# Default parameters
: ${rootfs_image_type:=tarball}

analyze_fstab() {
	if [ ! -s $rootfsDir/etc/fstab ]; then
		echo "No fstab file to analyze in '$rootfsDir/etc/fstab'"
		exit 3
	fi
	echo "Fstab analysis not yet implemented"
	exit 3
}

generate_tarball() {
	local imgRoot="${1:-$rootfsDir}"
	local imgName="$2"
	if [ -z "$imgName" ]; then
		imgName=$(basename $imgRoot)
	fi
	echo "tar cjf $imagesDir/$imgName.tar.bz2 -C $imgRoot ."
	tar cjf $imagesDir/$imgName.tar.bz2 -C $imgRoot .
}

generate_ext() {
	local imgRoot="$1"
	local imgType="${2:-ext4}"
	local imgName="$3"
	echo "Generation of ${imgType^^} images is not yet supported"
	exit 2
}

generate_image() {
	local imgRoot="$1"
	local imgType="${2:-tarball}"
	local imgName="$3"

	if [ -z "$imgName" ]; then
		imgName=$(basename $imgRoot)
	fi
	case $imgType in
		tarball)	# Generate a global tarball image of the rootfs
			generate_tarball $imgRoot $imgName
			;;

		ext[234])	# Generate a global EXTn image of the rootfs
			generate_ext $imgRoot $imgType $imgName
			;;
		*)
			echo "Unsupported image format '$imgType'"
			exit 1
			;;
	esac
}

if [ "$rootfs_image_type" = "splitted" ]; then
	# Analyze the /etc/fstab file and generate appropriatly formatted images
	mkdir $tmpDir
	tar cf - -C $outputDir rootfs | tar xf - -C $tmpDir
	tmpRoot=$tmpDir/rootfs
	# Splitted image may be a list of specifications rootDir:format:name
	# 	If format is omitted, tarball is assumed
	# 	If name is omitted, the basename of the directory is used (rootfs for /)
	splitted_images=${splitted_images:-$(analyze_fstab)}
	if [ -z "$splitted_images" ]; then
		echo "No image splitting defined... Can't generate splitted image"
		exit 3
	fi
	for image_spec in splitted_images; do
		echo $image_spec 
	done | sed -e 's/:/ /' | sort -r -k 1 | uniq | while read imgRoot imgType imgName; do
		generate_image $tmpRoot/$imgRoot $imgType $imgName
		rm -rf $tmpRoot/$imgRoot/* $tmpRoot/$imgRoot/.[^.]* $tmpRoot/$imgRoot/..?*
	done
else
	generate_image "$rootfsDir" "$rootfs_image_type"
fi

